"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PathResolver = void 0;
const entry_1 = require("./entry");
class PathResolver {
    opt;
    entries;
    constructor(entries, opt = { isDebug: false }) {
        this.opt = opt;
        this.entries = entries.map(x => entry_1.Entry.of(entry_1.Entry.cloneOrig(x)));
    }
    resolve(pathname) {
        if (!pathname.startsWith('/')) {
            console.error('pathname must start with the character "/":', pathname);
            throw new Error('pathname must start with the character "/"');
        }
        if (pathname.length > 1 && pathname.endsWith('/'))
            pathname = pathname.slice(0, -1); // remove the slash from the end
        const segments = pathname.split('/').slice(1); // "/hello/world" -> ["hello","world"]
        const req = {
            pathname,
            segments,
            level: segments.length - 1,
        };
        this.log(`resolving "${pathname}"`);
        const found = this.find(req, 0, this.entries);
        if (found)
            return {
                entry: found.entry.clone(),
                pathname,
                pathnameParams: found.pathnameParams,
                canActivateEntries: this.getCanActivateEntries(found.entry).map(x => x.clone()),
            };
    }
    find(req, level, entries) {
        if (!entries)
            return;
        for (let i = 0; i < entries.length; i++) {
            const entry = entries[i];
            if (entry.isSimpleSegment && entry.segment !== req.segments[level]) {
                this.logSkip('!= segment', entry);
                continue;
            }
            if (level === req.level || entry.isWildcard) {
                const match = entry.match(req.pathname);
                this.logMatch(match, entry);
                if (match) {
                    if (entry.resultIsOnlyChildren)
                        this.logSkip('result-is-only-children', entry);
                    else
                        return { entry, pathnameParams: match.params };
                }
            }
            else
                this.logSkip('skip-check', entry);
            const nextLevel = level + 1;
            if (nextLevel > req.level) {
                this.logSkip('skip-children', entry);
                continue;
            }
            const found = this.find(req, nextLevel, entry.children);
            if (found)
                return found;
        }
    }
    getCanActivateEntries(entry) {
        const result = [];
        if (entry.canActivate)
            result.push(entry);
        if (entry.parent)
            result.unshift(...this.getCanActivateEntries(entry.parent));
        return result;
    }
    //region Support
    logSkip(reason, entry) {
        if (this.opt.isDebug)
            console.log(` [x] ${entry.pathTemplate}, ${reason}`);
    }
    logMatch(match, entry) {
        if (this.opt.isDebug)
            console.log(` [${match ? 'v' : 'x'}] ${entry.pathTemplate}`);
    }
    log(data) {
        if (this.opt.isDebug)
            console.log(data);
    }
}
exports.PathResolver = PathResolver;
